package trail.slsb;

import javax.annotation.Resource;
import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;

public class AdministrationFacadeBeanInterceptor
{

	@Resource
	private javax.ejb.SessionContext ctx;

	@AroundInvoke
	public Object logMethodCall(InvocationContext ic) throws Exception
	{
		{
			// This block of code is executed before the EJB method
			long startTime = System.currentTimeMillis();
			// This block of code is executed before the EJB method
            String remoteClient = getCurrentClientIpAddress();
            
           
			String className = ic.getTarget().getClass().getName();
			String methodName = ic.getMethod().getName();
			Object[] parameters = ic.getParameters();
            
            
            System.out.println("Before: "+this.getLogString(null, className, methodName, remoteClient, parameters));
			
			
			try
			{
				// This block of code calls the EJB method (or the next
				// interceptor)
				return ic.proceed();
				// This block of code calls the EJB method (or the next
				// interceptor)
			} catch (Exception ex)
			{
				throw ex;
			} finally
			{
				
				long endTime = System.currentTimeMillis();
			    Long methodTime = startTime - endTime;
				System.out.println("After: "+getLogString(methodTime, className,
						methodName,remoteClient, parameters));
				
				// This block of code is executed after the EJB method
			}
			
			

		}

	}

	private String getCurrentClientIpAddress()
	{
		
		String currentThreadName = Thread.currentThread().getName();
		System.out.println("Threadname: "+currentThreadName);
		int begin = currentThreadName.indexOf('[') +1;
		int end = currentThreadName.indexOf(']')-1;
		String remoteClient = currentThreadName.substring(begin, end);
		return remoteClient;
	}

	// Convert a className, methodName and set of parameters into a nicely
	// formatted String
	private String getLogString(Long methodTime, String className,
			String methodName, String remoteClient, Object[] parameters)
	{
		StringBuffer logString = new StringBuffer();

		logString.append(remoteClient +" [" + className + "] " + methodName + "(");
		if (parameters != null)
		{
			for (int i = 0; i < parameters.length; i++)
			{
				logString.append(parameters[i]);
				if (i < parameters.length - 1)
				{
					logString.append(", ");
				}
			}
		}
		logString.append(")");
		if(methodTime!=null)
			logString.append(" - " + methodTime + "ms");

		return logString.toString();
	}
}